@@ -23,9 +23,15 @@
 
 #include "audio-resampler-x86-sse41.h"
 
-#ifdef _MSC_VER
-#include <intrin.h>             // _mm_cvtsi128_si64
+#if 0
+#define __SSE4_1__
+#pragma GCC target("sse4.1")
 #endif
+
+#if defined (__x86_64__) && \
+    defined (HAVE_SMMINTRIN_H) && defined (HAVE_EMMINTRIN_H) && \
+    defined (__SSE4_1__)
+
 #include <emmintrin.h>
 #include <smmintrin.h>
 
@@ -64,7 +70,7 @@ inner_product_gint32_full_1_sse41 (gint32 * o, const gint32 * a,
   res = _mm_cvtsi128_si64 (sum);
 
   res = (res + (1 << (PRECISION_S32 - 1))) >> PRECISION_S32;
-  *o = CLAMP (res, G_MININT32, G_MAXINT32);
+  *o = CLAMP (res, -(1L << 31), (1L << 31) - 1);
 }
 
 static inline void
@@ -107,7 +113,7 @@ inner_product_gint32_linear_1_sse41 (gint32 * o, const gint32 * a,
   res = _mm_cvtsi128_si64 (sum[0]);
 
   res = (res + (1 << (PRECISION_S32 - 1))) >> PRECISION_S32;
-  *o = CLAMP (res, G_MININT32, G_MAXINT32);
+  *o = CLAMP (res, -(1L << 31), (1L << 31) - 1);
 }
 
 static inline void
@@ -172,9 +178,11 @@ inner_product_gint32_cubic_1_sse41 (gint32 * o, const gint32 * a,
   res = _mm_cvtsi128_si64 (sum[0]);
 
   res = (res + (1 << (PRECISION_S32 - 1))) >> PRECISION_S32;
-  *o = CLAMP (res, G_MININT32, G_MAXINT32);
+  *o = CLAMP (res, -(1L << 31), (1L << 31) - 1);
 }
 
 MAKE_RESAMPLE_FUNC (gint32, full, 1, sse41);
 MAKE_RESAMPLE_FUNC (gint32, linear, 1, sse41);
 MAKE_RESAMPLE_FUNC (gint32, cubic, 1, sse41);
+
+#endif
