@@ -789,6 +789,8 @@ gst_audio_encoder_finish_frame (GstAudioEncoder * enc, GstBuffer * buf,
         ret = GST_FLOW_FLUSHING;
       else
         ret = GST_FLOW_NOT_NEGOTIATED;
+      if (buf)
+        gst_buffer_unref (buf);
       goto exit;
     }
   }
@@ -919,7 +921,12 @@ gst_audio_encoder_finish_frame (GstAudioEncoder * enc, GstBuffer * buf,
         priv->bytes_out += size;
         GST_OBJECT_UNLOCK (enc);
 
-        gst_pad_push (enc->srcpad, tmpbuf);
+        ret = gst_pad_push (enc->srcpad, tmpbuf);
+        if (ret != GST_FLOW_OK) {
+          GST_WARNING_OBJECT (enc, "pushing header returned %s",
+            gst_flow_get_name (ret));
+          goto exit;
+        }
       }
       priv->ctx.new_headers = FALSE;
     }
