@@ -527,7 +527,7 @@ _event_probe (GstPad * pad, GstPadProbeInfo * info, PrivateStream * ps)
 static GstStaticCaps subtitle_caps = GST_STATIC_CAPS ("text/x-raw; "
     "subpicture/x-pgs; subpicture/x-dvb; subpicture/x-dvd; "
     "application/x-subtitle-unknown; application/x-ssa; application/x-ass; "
-    "subtitle/x-kate; application/x-kate");
+    "subtitle/x-kate; application/x-kate; subpicture/x-xsub");
 
 static gboolean
 is_subtitle_caps (const GstCaps * caps)
@@ -1388,6 +1388,7 @@ handle_message (GstDiscoverer * dc, GstMessage * msg)
          */
         if (dc->priv->current_info->misc)
           gst_structure_free (dc->priv->current_info->misc);
+        dc->priv->current_info->misc = gst_structure_copy (structure);
         g_ptr_array_add (dc->priv->current_info->missing_elements_details,
             gst_missing_plugin_message_get_installer_detail (msg));
       } else if (sttype == _STREAM_TOPOLOGY_QUARK) {
@@ -1962,6 +1963,8 @@ gst_discoverer_start (GstDiscoverer * discoverer)
   GSource *source;
   GMainContext *ctx = NULL;
 
+  g_return_if_fail (GST_IS_DISCOVERER (discoverer));
+
   GST_DEBUG_OBJECT (discoverer, "Starting...");
 
   if (discoverer->priv->async) {
@@ -1999,6 +2002,8 @@ gst_discoverer_start (GstDiscoverer * discoverer)
 void
 gst_discoverer_stop (GstDiscoverer * discoverer)
 {
+  g_return_if_fail (GST_IS_DISCOVERER (discoverer));
+
   GST_DEBUG_OBJECT (discoverer, "Stopping...");
 
   if (!discoverer->priv->async) {
@@ -2064,6 +2069,8 @@ gst_discoverer_discover_uri_async (GstDiscoverer * discoverer,
 {
   gboolean can_run;
 
+  g_return_val_if_fail (GST_IS_DISCOVERER (discoverer), FALSE);
+
   GST_DEBUG_OBJECT (discoverer, "uri : %s", uri);
 
   DISCO_LOCK (discoverer);
@@ -2101,6 +2108,9 @@ gst_discoverer_discover_uri (GstDiscoverer * discoverer, const gchar * uri,
   GstDiscovererResult res = 0;
   GstDiscovererInfo *info;
 
+  g_return_val_if_fail (GST_IS_DISCOVERER (discoverer), NULL);
+  g_return_val_if_fail (uri, NULL);
+
   GST_DEBUG_OBJECT (discoverer, "uri:%s", uri);
 
   DISCO_LOCK (discoverer);
@@ -2191,9 +2201,14 @@ gst_discoverer_info_to_variant (GstDiscovererInfo * info,
   /* FIXME: implement TOC support */
   GVariant *stream_variant;
   GVariant *variant;
-  GstDiscovererStreamInfo *sinfo = gst_discoverer_info_get_stream_info (info);
+  GstDiscovererStreamInfo *sinfo;
   GVariant *wrapper;
 
+  g_return_val_if_fail (GST_IS_DISCOVERER_INFO (info), NULL);
+  g_return_val_if_fail (gst_discoverer_info_get_result (info) ==
+      GST_DISCOVERER_OK, NULL);
+
+  sinfo = gst_discoverer_info_get_stream_info (info);
   stream_variant = gst_discoverer_info_to_variant_recurse (sinfo, flags);
   variant =
       g_variant_new ("(vv)", _serialize_info (info, flags), stream_variant);
